@@ -1,17 +1,19 @@
-"""@namespace stomp \mainpage
-stomp.py provides connectivity to a message broker supporting the STOMP protocol. Protocol versions 1.0, 1.1 and 1.2 are supported.
+"""stomp.py provides connectivity to a message broker supporting the STOMP protocol.
+Protocol versions 1.0, 1.1 and 1.2 are supported.
 
 See the GITHUB project page for more information.
 
-Author: Jason R Briggs<br />
-License: http://www.apache.org/licenses/LICENSE-2.0<br />
+Author: Jason R Briggs
+License: http://www.apache.org/licenses/LICENSE-2.0
 Project Page: https://github.com/jasonrbriggs/stomp.py
 
 """
 
-import stomp.connect as connect, stomp.listener as listener
+import stomp.connect as connect
+import stomp.listener as listener
+import stomp.adapter as adapter
 
-__version__ = (4, 1, 2)
+__version__ = (4, 1, 24)
 
 ##
 # Alias for STOMP 1.0 connections.
@@ -49,3 +51,7 @@ StatsListener = listener.StatsListener
 ##
 # Access to the 'waiting' listener.
 WaitingListener = listener.WaitingListener
+
+##
+# Access to the printing listener
+PrintingListener = listener.PrintingListener
