@@ -1,34 +1,60 @@
+"""
+Stomp.py command-line client
+
+Usage: stomp [options]
+
+Options:
+  --version                    Show the version number and exit
+  -h, --help                   Show this help message and exit
+  -H <host>, --host=<port>     Hostname or IP address to connect to. [default: localhost]
+  -P <port>, --port=<port>     Port providing stomp protocol connections. [default: 61613]
+  -U <user>, --user=<user>     Username for the connection
+  -W <password>, --password=<password>
+                               Password for the connection
+  -F <filename>, --file=<filename>
+                               File containing commands to be executed, instead of
+                               prompting from the command prompt.
+  -S <protocol version>, --protocol=<protocol version>
+                               Set the STOMP protocol version (1.0, 1.1, 1.2) [default: 1.1]
+  -L <queue>, --listen=<queue> Listen for messages on a queue/destination
+  -V, --verbose                Verbose logging "on" or "off" (if on, full headers
+                               from stomp server responses are printed)
+  --heartbeats=<heartbeats>    Heartbeats to request when connecting with protocol >=
+                               1.1 (two comma separated integers required) [default: 0,0]
+  --ssl                        Enable SSL connection
+  --ssl-key-file=<key-file>    ssl key file
+  --ssl-cert-file=<cert-file>  ssl cert file
+  --ssl-ca-file=<ca-file>      ssl ca certs file
+
+"""
+
 import base64
+from cmd import Cmd
+import json
 import os
+import re
 import sys
 import time
+from functools import partial
 
-from cmd import Cmd
-from optparse import OptionParser
+from docopt import docopt
 
-from stomp.connect import StompConnection10,StompConnection11,StompConnection12
-from stomp.listener import ConnectionListener, StatsListener
+sys.path.append(os.path.dirname(os.path.dirname(os.path.realpath(__file__))))
 from stomp.adapter.multicast import MulticastConnection
 import stomp.colors
+import stomp.utils
+from stomp.connect import StompConnection10, StompConnection11, StompConnection12
+from stomp.listener import ConnectionListener, StatsListener
+
 sys.path.append('.')
 import stomp
 
+version_string = '%s.%s.%s' % stomp.__version__
 
-##@namespace stomp.__main__
-# The stomp.py command line client (used for testing or simple STOMP command scripting).
-
-##
-# Command-line version string
-#
-stomppy_version = 'Stomp.py Version %s.%s.%s' % stomp.__version__
-
-try:
-    import uuid
-except ImportError:
-    from backward import uuid
+heartbeat_pattern = re.compile(r'[0-9]+,[0-9]+')
 
 
-class SubscriptionInfo:
+class SubscriptionInfo(object):
     """
     Used to store info about a subscription.
     """
@@ -36,29 +62,34 @@ class SubscriptionInfo:
         self.id = id
         self.ack = ack
 
+
 class StompCLI(Cmd, ConnectionListener):
     """
-    A command line interface to the stomp.py client.  See \link stomp::connect::StompConnection11 \endlink
+    A command line interface to the stomp.py client.  See :py:class:`stomp.connect.StompConnection11`
     for more information on establishing a connection to a stomp server.
     """
-    def __init__(self, host='localhost', port=61613, user='', passcode='', ver='1.1', prompt='> ', verbose=True, stdin=sys.stdin, stdout=sys.stdout):
+    def __init__(self, host='localhost', port=61613, user='', passcode='', ver='1.1', prompt='> ', verbose=True,
+                 heartbeats=(0, 0), use_ssl=False, ssl_key_file=None, ssl_cert_file=None, ssl_ca_file=None, stdin=sys.stdin, stdout=sys.stdout):
         Cmd.__init__(self, 'Tab', stdin, stdout)
         ConnectionListener.__init__(self)
+        self.__start = True
         self.prompt = prompt
         self.verbose = verbose
         self.user = user
         self.passcode = passcode
         self.__quit = False
         if ver == '1.0':
-            self.conn = StompConnection10([(host, port)], wait_on_receipt=True)
+            self.conn = StompConnection10([(host, port)])
         elif ver == '1.1':
-            self.conn = StompConnection11([(host, port)], wait_on_receipt=True)
+            self.conn = StompConnection11([(host, port)], heartbeats=heartbeats)
         elif ver == '1.2':
-            self.conn = StompConnection12([(host, port)], wait_on_receipt=True)
+            self.conn = StompConnection12([(host, port)], heartbeats=heartbeats)
         elif ver == 'multicast':
             self.conn = MulticastConnection()
         else:
             raise RuntimeError('Unknown version')
+        if use_ssl:
+            self.conn.set_ssl([(host, port)], key_file=ssl_key_file, cert_file=ssl_cert_file, ca_certs=ssl_ca_file)
         self.conn.set_listener('', self)
         self.conn.start()
         self.conn.connect(self.user, self.passcode, wait=True)
@@ -76,18 +107,29 @@ class StompCLI(Cmd, ConnectionListener):
         Utility function to print a message and setup the command prompt
         for the next input
         """
-        self.__sysout("\r  \r", end='')
+        if self.__quit:
+            return
         if self.verbose:
             self.__sysout(frame_type)
-            for header_key in headers.keys():
-                self.__sysout('%s: %s' % (header_key, headers[header_key]))
+            for k, v in headers.items():
+                self.__sysout('%s: %s' % (k, v))
+        else:
+            if 'message-id' in headers:
+                self.__sysout('message-id: %s' % headers['message-id'])
+            if 'subscription' in headers:
+                self.__sysout('subscription: %s' % headers['subscription'])
         if self.prompt != '':
             self.__sysout('')
         self.__sysout(body)
-        self.__sysout(self.prompt, end='')
+        if not self.__start:
+            self.__sysout(self.prompt, end='')
+        else:
+            self.__start = False
         self.stdout.flush()
 
     def __sysout(self, msg, end="\n"):
+        if self.__quit:
+            return
         self.stdout.write(str(msg) + end)
 
     def __error(self, msg, end="\n"):
@@ -95,54 +137,59 @@ class StompCLI(Cmd, ConnectionListener):
 
     def on_connecting(self, host_and_port):
         """
-        \see ConnectionListener::on_connecting
+        See :py:meth:`ConnectionListener.on_connecting`
         """
+        pass
 
     def on_disconnected(self):
         """
-        \see ConnectionListener::on_disconnected
+        see :py:meth:`ConnectionListener.on_disconnected`
         """
         if not self.__quit:
             self.__error("lost connection")
 
     def on_message(self, headers, body):
         """
-        \see ConnectionListener::on_message
+        See :py:meth:`ConnectionListener.on_message`
 
         Special case: if the header 'filename' is present, the content is written out
         as a file
         """
+        self.__sysout('')
         if 'filename' in headers:
             content = base64.b64decode(body.encode())
             if os.path.exists(headers['filename']):
                 fname = '%s.%s' % (headers['filename'], int(time.time()))
             else:
                 fname = headers['filename']
-            f = open(fname, 'wb')
-            f.write(content)
-            f.close()
+            with open(fname, 'wb') as f:
+                f.write(content)
             self.__print_async("MESSAGE", headers, "Saved file: %s" % fname)
         else:
             self.__print_async("MESSAGE", headers, body)
 
     def on_error(self, headers, body):
         """
-        \see ConnectionListener::on_error
+        See :py:meth:`ConnectionListener.on_error`
         """
         self.__print_async("ERROR", headers, body)
 
     def on_receipt(self, headers, body):
         """
-        \see ConnectionListener::on_receipt
+        See :py:meth:`ConnectionListener.on_receipt`
         """
         self.__print_async("RECEIPT", headers, body)
 
     def on_connected(self, headers, body):
         """
-        \see ConnectionListener::on_connected
+        See :py:meth:`ConnectionListener.on_connected`
         """
         self.__print_async("CONNECTED", headers, body)
 
+    def on_send(self, frame):
+        if self.verbose:
+            self.__sysout("Sending %s" % str(frame))
+
     def help_help(self):
         self.__sysout('Quick help on commands')
 
@@ -152,20 +199,17 @@ class StompCLI(Cmd, ConnectionListener):
     def emptyline(self):
         pass
 
-    def help(self, usage, description, required=[], optional=[]):
-        required.insert(0, '')
-        rparams = "\n\t".join(required)
-
-        optional.insert(0, '')
-        oparams = "\n\t".join(optional)
+    def help(self, usage, description, required=(), optional=()):
+        rparams = "\n\t" + "\n\t".join(required)
+        oparams = "\n\t" + "\n\t".join(optional)
 
         m = {
-            'hl' : stomp.colors.BOLD + stomp.colors.GREEN,
-            'nc' : stomp.colors.NO_COLOR,
-            'usage' : usage,
-            'description' : description,
-            'required' : rparams.rstrip(),
-            'optional' : oparams.rstrip()
+            'hl': stomp.colors.BOLD + stomp.colors.GREEN,
+            'nc': stomp.colors.NO_COLOR,
+            'usage': usage,
+            'description': description,
+            'required': rparams.rstrip(),
+            'optional': oparams.rstrip()
         }
 
         if rparams.rstrip() != '':
@@ -185,6 +229,7 @@ class StompCLI(Cmd, ConnectionListener):
 
     def do_quit(self, args):
         self.__quit = True
+        self.__sysout('Shutting down, please wait')
         return True
     do_exit = do_quit
     do_EOF = do_quit
@@ -194,7 +239,7 @@ class StompCLI(Cmd, ConnectionListener):
     help_exit = help_quit
 
     def help_EOF(self):
-        self.__sysout('')
+        self.help('exit', 'Exit the stomp client (using CTRL-D)')
 
     def do_subscribe(self, args):
         args = args.split()
@@ -220,9 +265,9 @@ class StompCLI(Cmd, ConnectionListener):
 
     def help_subscribe(self):
         self.help('subscribe <destination> [ack]',
-            '''Register to listen to a given destination. Like send, the subscribe command requires a destination
+                  '''Register to listen to a given destination. Like send, the subscribe command requires a destination
 \theader indicating which destination to subscribe to. The ack parameter is optional, and defaults to
-\tauto.''', [ 'destination - the name to subscribe to' ], [ 'ack - how to handle acknowledgements for a message; either automatically (auto) or manually (client)' ])
+\tauto.''', ['destination - the name to subscribe to'], ['ack - how to handle acknowledgements for a message; either automatically (auto) or manually (client)'])
 
     def do_unsubscribe(self, args):
         args = args.split()
@@ -230,7 +275,7 @@ class StompCLI(Cmd, ConnectionListener):
             self.__error('Expecting: unsubscribe <destination>')
             return
 
-        if args[0] not in self.__subscriptions.keys():
+        if args[0] not in self.__subscriptions:
             self.__sysout('Subscription %s not found' % args[0])
             return
 
@@ -240,7 +285,7 @@ class StompCLI(Cmd, ConnectionListener):
 
     def help_unsubscribe(self):
         self.help('unsubscribe <destination>', 'Remove an existing subscription - so that the client no longer receive messages from that destination.',
-                [ 'destination - the name to unsubscribe from' ], [ 'ack - how to handle acknowledgements for a message; either automatically (auto) or manually (client)' ])
+                  ['destination - the name to unsubscribe from'], ['ack - how to handle acknowledgements for a message; either automatically (auto) or manually (client)'])
 
     def do_send(self, args):
         args = args.split()
@@ -262,11 +307,11 @@ class StompCLI(Cmd, ConnectionListener):
 
     def help_send(self):
         self.help('send <destination> <message>', 'Sends a message to a destination in the messaging system.',
-            [ 'destination - where to send the message', 'message - the content to send' ])
+                  ['destination - where to send the message', 'message - the content to send'])
 
     def do_sendrec(self, args):
         args = args.split()
-        receipt_id = str(uuid.uuid4())
+        receipt_id = stomp.utils.get_uuid()
         if len(args) < 2:
             self.__error('Expecting: sendrec <destination> <message>')
         elif not self.transaction_id:
@@ -275,8 +320,9 @@ class StompCLI(Cmd, ConnectionListener):
             self.conn.send(args[0], ' '.join(args[1:]), transaction=self.transaction_id, receipt=receipt_id)
 
     def help_sendrec(self):
-        self.help('sendrec <destination> <message>', 'Sends a message to a destination in the messaging system and blocks for receipt of the message.',
-                    [ 'destination - where to send the message', 'message - the content to send' ])
+        self.help('sendrec <destination> <message>',
+                  'Sends a message to a destination in the messaging system and blocks for receipt of the message.',
+                  ['destination - where to send the message', 'message - the content to send'])
 
     def do_sendreply(self, args):
         args = args.split()
@@ -286,36 +332,53 @@ class StompCLI(Cmd, ConnectionListener):
             self.conn.send(args[0], "%s\n" % ' '.join(args[2:]), headers={'correlation-id': args[1]})
 
     def help_sendreply(self):
-        self.help('sendreply <destination> <correlation-id> <message>', 'Sends a reply message to a destination in the messaging system.',
-                [ 'destination - where to send the message', 'correlation-id - the correlating identifier to send with the response', 'message - the content to send' ])
+        self.help('sendreply <destination> <correlation-id> <message>',
+                  'Sends a reply message to a destination in the messaging system.',
+                  ['destination - where to send the message',
+                   'correlation-id - the correlating identifier to send with the response',
+                   'message - the content to send'])
 
     def do_sendfile(self, args):
         args = args.split()
         if len(args) < 2:
-            self.__error('Expecting: sendfile <destination> <filename>')
+            self.__error('Expecting: sendfile <destination> <filename> [headers.json]')
         elif not os.path.exists(args[1]):
             self.__error('File %s does not exist' % args[1])
         else:
-            s = open(args[1], mode='rb').read()
+            headers = {}
+            if len(args) == 3:
+                if not os.path.exists(args[2]):
+                    self.__error('File %s does not exist' % args[2])
+                    return
+                self.__sysout("Loading %s" % args[2])
+                with open(args[2], mode='rb') as jf:
+                    headers = json.load(jf)
+                    self.__sysout('Using headers %s' % str(headers))
+
+            with open(args[1], mode='rb') as f:
+                s = f.read()
             msg = base64.b64encode(s).decode()
             if not self.transaction_id:
-                self.conn.send(args[0], msg, filename=args[1])
+                self.conn.send(args[0], msg, filename=args[1], headers=headers)
             else:
-                self.conn.send(args[0], msg, filename=args[1], transaction=self.transaction_id)
+                self.conn.send(args[0], msg, filename=args[1], headers=headers, transaction=self.transaction_id)
 
     def help_sendfile(self):
-        self.help('sendfile <destination> <filename>', 'Sends a file to a destination in the messaging system.',
-                [ 'destination - where to send the message', 'filename - the file to send' ])
+        self.help('sendfile <destination> <filename> [headers.json]',
+                  'Sends a file to a destination in the messaging system.',
+                  ['destination - where to send the message', 'filename - the file to send',
+                   'headers.json - json map with headers to send'])
 
     def do_version(self, args):
-        self.__sysout('%s%s [Protocol version %s]%s' % (stomp.colors.BOLD, stomppy_version, self.conn.version, stomp.colors.NO_COLOR))
+        self.__sysout('%s%s [Protocol version %s]%s' %
+                      (stomp.colors.BOLD, version_string, self.conn.version, stomp.colors.NO_COLOR))
     do_ver = do_version
 
     def help_version(self):
         self.help('version', 'Display the version of the client')
     help_ver = help_version
 
-    def check_ack_nack(self, cmd, args):
+    def check_ack_nack(self, acknackfunc, args):
         if self.nversion >= 1.2 and len(args) < 1:
             self.__error("Expecting: %s <ack-id>" % cmd)
             return None
@@ -326,52 +389,50 @@ class StompCLI(Cmd, ConnectionListener):
             self.__error("Expecting: %s <message-id>" % cmd)
             return None
 
-        if len(args) == 1:
-            return (args[0], None)
+        if self.nversion == 1.1:
+            return partial(acknackfunc, args[0], args[1])
         else:
-            return (args[0], args[1])
+            return partial(acknackfunc, args[0])
 
     def do_ack(self, args):
         args = args.split()
-        hdrs = self.check_ack_nack('ack', args)
-        if hdrs is None:
+        func = self.check_ack_nack(self.conn.ack, args)
+        if func is None:
             return
 
-        (message_id, subscription_id) = hdrs
-
         if not self.transaction_id:
-            self.conn.ack(message_id, subscription_id)
+            func()
         else:
-            self.conn.ack(message_id, subscription_id, transaction=self.transaction_id)
+            func(transaction=self.transaction_id)
 
     def help_ack(self):
         self.help('ack <message-id> [subscription-id]', '''The command 'ack' is used to acknowledge consumption of a message from a subscription using client
 \tacknowledgment. When a client has issued a 'subscribe' with the ack flag set to client, any messages
 \treceived from that destination will not be considered to have been consumed (by the server) until
-\tthe message has been acknowledged.''', [ 'message-id - the id of the message being acknowledged' ], [ 'subscription-id the id of the subscription (only required for STOMP 1.1)' ] )
+\tthe message has been acknowledged.''', ['message-id - the id of the message being acknowledged'], ['subscription-id the id of the subscription (only required for STOMP 1.1)'])
 
     def do_nack(self, args):
         args = args.split()
-        hdrs = self.check_ack_nack('nack', args)
-        if hdrs is None:
+        func = self.check_ack_nack(self.conn.nack, args)
+        if func is None:
             return
 
         if not self.transaction_id:
-            self.conn.nack(headers = hdrs)
+            func()
         else:
-            self.conn.nack(headers = hdrs, transaction=self.transaction_id)
+            func(transaction=self.transaction_id)
 
     def help_nack(self):
         self.help('nack <message-id> [subscription]', '''The command 'nack' is used to acknowledge the failure of a message from a subscription using client
 \tacknowledgment. When a client has issued a 'subscribe' with the ack flag set to client, any messages
 \treceived from that destination will not be considered to have been consumed (by the server) until
-\tthe message has been acknowledged (ack or nack).''', [ 'message-id - the id of the message being acknowledged' ])
+\tthe message has been acknowledged (ack or nack).''', ['message-id - the id of the message being acknowledged'])
 
     def do_abort(self, args):
         if not self.transaction_id:
             self.__error("Not currently in a transaction")
         else:
-            self.conn.abort(transaction = self.transaction_id)
+            self.conn.abort(transaction=self.transaction_id)
             self.__sysout('Aborted transaction: %s' % self.transaction_id)
             self.transaction_id = None
     do_rollback = do_abort
@@ -429,67 +490,74 @@ class StompCLI(Cmd, ConnectionListener):
         elif not os.path.exists(args[0]):
             self.__error("File %s was not found" % args[0])
         else:
-            lines = open(args[0]).read().split('\n')
+            with open(args[0]) as f:
+                lines = f.read().split('\n')
             for line in lines:
                 self.onecmd(line)
 
     def help_run(self):
         self.help('run <filename>', 'Execute commands in a specified file')
 
+
 def do_nothing_loop():
     while 1:
         time.sleep(1)
 
+
+def optional_arg(arg_default):
+    def func(option, opt_str, value, parser):
+        if parser.rargs and not parser.rargs[0].startswith('-'):
+            val = parser.rargs[0]
+            parser.rargs.pop(0)
+        else:
+            val = arg_default
+        setattr(parser.values, option.dest, val)
+    return func
+
+
 def main():
-    parser = OptionParser(version=stomppy_version)
+    arguments = docopt(__doc__, version=version_string)
 
-    parser.add_option('-H', '--host', type = 'string', dest = 'host', default = 'localhost',
-                      help = 'Hostname or IP to connect to. Defaults to localhost if not specified.')
-    parser.add_option('-P', '--port', type = int, dest = 'port', default = 61613,
-                      help = 'Port providing stomp protocol connections. Defaults to 61613 if not specified.')
-    parser.add_option('-U', '--user', type = 'string', dest = 'user', default = None,
-                      help = 'Username for the connection')
-    parser.add_option('-W', '--password', type = 'string', dest = 'password', default = None,
-                      help = 'Password for the connection')
-    parser.add_option('-F', '--file', type = 'string', dest = 'filename',
-                      help = 'File containing commands to be executed, instead of prompting from the command prompt.')
-    parser.add_option('-S', '--stomp', type = 'string', dest = 'stomp', default = '1.1',
-                      help = 'Set the STOMP protocol version.')
-    parser.add_option('-L', '--listen', type = 'string', dest = 'listen', default = None,
-                      help = 'Listen for messages on a queue/destination')
-    parser.add_option("-V", "--verbose", dest = "verbose", default = 'on',
-                      help = 'Verbose logging "on" or "off" (if on, full headers from stomp server responses are printed)')
-
-    parser.set_defaults()
-    (options, args) = parser.parse_args()
-
-    if options.verbose == 'on':
-        verbose = True
-    else:
-        verbose = False
-
-    if options.listen:
+    if arguments['--listen'] is not None:
         prompt = ''
     else:
         prompt = '> '
 
-    st = StompCLI(options.host, options.port, options.user, options.password, options.stomp, prompt, verbose)
+    if not heartbeat_pattern.match(arguments['--heartbeats']):
+        print('Invalid heartbeats, expecting cx,cy')
+        sys.exit(1)
 
-    if options.listen:
-        st.do_subscribe(options.listen)
+    heartbeats = tuple(map(int, arguments['--heartbeats'].split(",")))
+
+    st = StompCLI(arguments['--host'], arguments['--port'], arguments['--user'], arguments['--password'], arguments['--protocol'],
+        prompt, arguments['--verbose'], heartbeats=heartbeats,
+        use_ssl=arguments['--ssl'],
+        ssl_key_file=arguments['--ssl-key-file'],
+        ssl_cert_file=arguments['--ssl-cert-file'],
+        ssl_ca_file=arguments['--ssl-ca-file'])
+
+    if arguments['--listen'] is not None:
+        st.do_subscribe(arguments['--listen'])
         try:
             while 1:
                 time.sleep(10)
         except:
             print("\n")
-    elif options.filename:
-        st.do_run(options.filename)
+    elif arguments['--file'] is not None:
+        st.do_run(arguments['--file'])
     else:
+        # disable CTRL-C, since can't guarantee correct handling of disconnect
+        import signal
+
+        def signal_handler(signal, frame):
+            pass
+        signal.signal(signal.SIGINT, signal_handler)
+
         try:
             try:
                 st.cmdloop()
             except KeyboardInterrupt:
-                pass
+                st.do_quit()
         finally:
             st.conn.disconnect()
 
@@ -498,4 +566,7 @@ def main():
 # command line access
 #
 if __name__ == '__main__':
-    main()
+    try:
+        main()
+    except:
+        pass
