@@ -5,18 +5,18 @@ import unittest
 
 import stomp
 from stomp import exception
-
+from stomp.backward import monotonic
+from stomp.listener import TestListener
 from stomp.test.testutils import *
 
 
 class TestBasicSend(unittest.TestCase):
 
     def setUp(self):
-        conn = stomp.Connection(get_standard_host())
+        conn = stomp.Connection(get_default_host())
         listener = TestListener('123')
         conn.set_listener('', listener)
-        conn.start()
-        conn.connect('admin', 'password', wait=True)
+        conn.connect(get_default_user(), get_default_password(), wait=True)
         self.conn = conn
         self.listener = listener
         self.timestamp = time.strftime('%Y%m%d%H%M%S')
@@ -31,11 +31,11 @@ class TestBasicSend(unittest.TestCase):
 
         self.conn.send(body='this is a test', destination=queuename, receipt='123')
 
-        self.listener.wait_on_receipt()
+        self.listener.wait_for_message()
 
-        self.assert_(self.listener.connections == 1, 'should have received 1 connection acknowledgement')
-        self.assert_(self.listener.messages == 1, 'should have received 1 message')
-        self.assert_(self.listener.errors == 0, 'should not have received any errors')
+        self.assertTrue(self.listener.connections == 1, 'should have received 1 connection acknowledgement')
+        self.assertTrue(self.listener.messages == 1, 'should have received 1 message')
+        self.assertTrue(self.listener.errors == 0, 'should not have received any errors')
 
     def test_commit(self):
         queuename = '/queue/test2-%s' % self.timestamp
@@ -47,14 +47,15 @@ class TestBasicSend(unittest.TestCase):
 
         time.sleep(3)
 
-        self.assert_(self.listener.connections == 1, 'should have received 1 connection acknowledgement')
-        self.assert_(self.listener.messages == 0, 'should not have received any messages')
+        self.assertTrue(self.listener.connections == 1, 'should have received 1 connection acknowledgement')
+        self.assertTrue(self.listener.messages == 0, 'should not have received any messages')
 
-        self.conn.commit(transaction = trans_id)
-        self.listener.wait_on_receipt()
+        self.conn.commit(transaction=trans_id)
+        self.listener.wait_for_message()
+        time.sleep(3)
 
-        self.assert_(self.listener.messages == 3, 'should have received 3 messages')
-        self.assert_(self.listener.errors == 0, 'should not have received any errors')
+        self.assertTrue(self.listener.messages == 3, 'should have received 3 messages')
+        self.assertTrue(self.listener.errors == 0, 'should not have received any errors')
 
     def test_abort(self):
         queuename = '/queue/test3-%s' % self.timestamp
@@ -66,27 +67,27 @@ class TestBasicSend(unittest.TestCase):
 
         time.sleep(3)
 
-        self.assert_(self.listener.connections == 1, 'should have received 1 connection acknowledgement')
-        self.assert_(self.listener.messages == 0, 'should not have received any messages')
+        self.assertTrue(self.listener.connections == 1, 'should have received 1 connection acknowledgement')
+        self.assertTrue(self.listener.messages == 0, 'should not have received any messages')
 
-        self.conn.abort(transaction = trans_id)
+        self.conn.abort(transaction=trans_id)
         time.sleep(3)
 
-        self.assert_(self.listener.messages == 0, 'should not have received any messages')
-        self.assert_(self.listener.errors == 0, 'should not have received any errors')
+        self.assertTrue(self.listener.messages == 0, 'should not have received any messages')
+        self.assertTrue(self.listener.errors == 0, 'should not have received any errors')
 
     def test_timeout(self):
         conn = stomp.Connection([('192.0.2.0', 60000)], timeout=5, reconnect_attempts_max=1)
         conn.set_listener('', self.listener)
 
         try:
-            ms = time.time()
-            conn.start()
+            ms = monotonic()
+            conn.connect("test", "test")
             self.fail("shouldn't happen")
         except exception.ConnectFailedException:
-            pass # success!
-            ms = time.time() - ms
-            self.assert_(ms > 5.0, 'connection timeout should have been at least 5 seconds')
+            pass  # success!
+            ms = monotonic() - ms
+            self.assertTrue(ms > 5.0, 'connection timeout should have been at least 5 seconds')
 
     def test_childinterrupt(self):
         def childhandler(signum, frame):
@@ -111,11 +112,11 @@ class TestBasicSend(unittest.TestCase):
 
         self.conn.send(body='this is an interrupt test 2', destination=queuename, receipt='123')
 
-        self.listener.wait_on_receipt()
+        self.listener.wait_for_message()
 
-        self.assert_(self.listener.connections == 1, 'should have received 1 connection acknowledgment')
-        self.assert_(self.listener.errors == 0, 'should not have received any errors')
-        self.assert_(self.conn.is_connected(), 'should still be connected to STOMP provider')
+        self.assertTrue(self.listener.connections == 1, 'should have received 1 connection acknowledgment')
+        self.assertTrue(self.listener.errors == 0, 'should not have received any errors')
+        self.assertTrue(self.conn.is_connected(), 'should still be connected to STOMP provider')
 
     def test_clientack(self):
         queuename = '/queue/testclientack-%s' % self.timestamp
@@ -123,9 +124,9 @@ class TestBasicSend(unittest.TestCase):
 
         self.conn.send(body='this is a test', destination=queuename, receipt='123')
 
-        self.listener.wait_on_receipt()
+        self.listener.wait_for_message()
 
-        (headers, msg) = self.listener.get_latest_message()
+        (headers, _) = self.listener.get_latest_message()
 
         message_id = headers['message-id']
         subscription = headers['subscription']
@@ -138,9 +139,9 @@ class TestBasicSend(unittest.TestCase):
 
         self.conn.send(body='this is a test', destination=queuename, receipt='123')
 
-        self.listener.wait_on_receipt()
+        self.listener.wait_for_message()
 
-        (headers, msg) = self.listener.get_latest_message()
+        (headers, _) = self.listener.get_latest_message()
 
         message_id = headers['message-id']
         subscription = headers['subscription']
@@ -152,31 +153,30 @@ class TestBasicSend(unittest.TestCase):
         self.conn.subscribe(destination=queuename, id=1, ack='client')
 
         hdrs = {
-            'special-1' : 'test with colon : test',
-            'special-2' : 'test with backslash \\ test',
-            'special-3' : 'test with newline \n'
+            'special-1': 'test with colon : test',
+            'special-2': 'test with backslash \\ test',
+            'special-3': 'test with newline \n'
         }
 
-        self.conn.send(body='this is a test', headers = hdrs, destination=queuename, receipt='123')
+        self.conn.send(body='this is a test', headers=hdrs, destination=queuename, receipt='123')
 
-        self.listener.wait_on_receipt()
+        self.listener.wait_for_message()
 
-        (headers, msg) = self.listener.get_latest_message()
+        (headers, _) = self.listener.get_latest_message()
 
-        message_id = headers['message-id']
-        subscription = headers['subscription']
-        self.assert_('special-1' in headers)
+        _ = headers['message-id']
+        _ = headers['subscription']
+        self.assertTrue('special-1' in headers)
         self.assertEqual('test with colon : test', headers['special-1'])
-        self.assert_('special-2' in headers)
+        self.assertTrue('special-2' in headers)
         self.assertEqual('test with backslash \\ test', headers['special-2'])
-        self.assert_('special-3' in headers)
+        self.assertTrue('special-3' in headers)
         self.assertEqual('test with newline \n', headers['special-3'])
 
 
 class TestConnectionErrors(unittest.TestCase):
     def test_connect_wait_error(self):
-        conn = stomp.Connection(get_standard_host())
-        conn.start()
+        conn = stomp.Connection(get_default_host())
         try:
             conn.connect('invalid', 'user', True)
             self.fail("Shouldn't happen")
@@ -184,10 +184,27 @@ class TestConnectionErrors(unittest.TestCase):
             pass
 
     def test_connect_nowait_error(self):
-        conn = stomp.Connection(get_standard_host())
-        conn.start()
+        conn = stomp.Connection(get_default_host())
         try:
             conn.connect('invalid', 'user', False)
-            self.assert_(conn.is_connected() == False, 'Should not be connected')
+            self.assertFalse(conn.is_connected(), 'Should not be connected')
         except:
             self.fail("Shouldn't happen")
+
+class TestContext(unittest.TestCase):
+    def setUp(self):
+        self.timestamp = time.strftime('%Y%m%d%H%M%S')
+
+    def test_with_context(self):
+        with stomp.Connection(get_default_host()) as conn:
+            self.listener = TestListener('123')
+            conn.set_listener('', self.listener)
+            conn.connect(get_default_user(), get_default_password(), wait=True)
+
+            queuename = '/queue/test1-%s' % self.timestamp
+            conn.subscribe(destination=queuename, id=1, ack='auto')
+            conn.send(body='this is a test', destination=queuename, receipt='123')
+            self.listener.wait_for_message()
+
+        self.assertTrue(self.listener.connections == 1, 'should have received 1 connection acknowledgement')
+        self.assertTrue(self.listener.disconnects >= 1, 'should have received 1 disconnect')
\ No newline at end of file
